@@ -167,7 +167,7 @@ int find_new_gid (bool sys_group,
 	bool *used_gids;
 	const struct group *grp;
 	gid_t gid_min, gid_max, preferred_min;
-	gid_t group_id, id;
+	gid_t id;
 	gid_t lowest_found, highest_found;
 	int result;
 	int nospam = 0;
