@@ -34,6 +34,7 @@
 
 #ident "$Id$"
 
+#include <assert.h>
 #include <errno.h>
 #include <fcntl.h>
 #include <getopt.h>
@@ -52,6 +53,7 @@
 #include "getdef.h"
 #include "groupio.h"
 #include "nscd.h"
+#include "sssd.h"
 #include "prototypes.h"
 #include "pwauth.h"
 #include "pwio.h"
@@ -95,7 +97,10 @@ static char *user_home;
 
 static bool fflg = false;
 static bool rflg = false;
+#ifdef WITH_SELINUX
 static bool Zflg = false;
+#endif
+static bool Rflg = false;
 
 static bool is_shadow_pwd;
 
@@ -113,6 +118,8 @@ static bool sub_uid_locked = false;
 static bool sub_gid_locked = false;
 #endif				/* ENABLE_SUBIDS */
 
+static const char* prefix = "";
+
 /* local function prototypes */
 static void usage (int status);
 static void update_groups (void);
@@ -149,6 +156,7 @@ static void usage (int status)
 	(void) fputs (_("  -h, --help                    display this help message and exit\n"), usageout);
 	(void) fputs (_("  -r, --remove                  remove home directory and mail spool\n"), usageout);
 	(void) fputs (_("  -R, --root CHROOT_DIR         directory to chroot into\n"), usageout);
+	(void) fputs (_("  -P, --prefix PREFIX_DIR       prefix directory where are located the /etc/* files\n"), usageout);
 #ifdef WITH_SELINUX
 	(void) fputs (_("  -Z, --selinux-user            remove any SELinux user mapping for the user\n"), usageout);
 #endif				/* WITH_SELINUX */
@@ -326,8 +334,8 @@ static void remove_usergroup (void)
 		 * Scan the passwd file to check if this group is still
 		 * used as a primary group.
 		 */
-		setpwent ();
-		while ((pwd = getpwent ()) != NULL) {
+		prefix_setpwent ();
+		while ((pwd = prefix_getpwent ()) != NULL) {
 			if (strcmp (pwd->pw_name, user_name) == 0) {
 				continue;
 			}
@@ -338,7 +346,7 @@ static void remove_usergroup (void)
 				break;
 			}
 		}
-		endpwent ();
+		prefix_endpwent ();
 	}
 
 	if (NULL == pwd) {
@@ -814,9 +822,10 @@ static int is_owner (uid_t uid, const char *path)
 static int remove_mailbox (void)
 {
 	const char *maildir;
-	char mailfile[1024];
+	char* mailfile;
 	int i;
 	int errors = 0;
+	size_t len;
 
 	maildir = getdef_str ("MAIL_DIR");
 #ifdef MAIL_SPOOL_DIR
@@ -827,13 +836,26 @@ static int remove_mailbox (void)
 	if (NULL == maildir) {
 		return 0;
 	}
-	snprintf (mailfile, sizeof mailfile, "%s/%s", maildir, user_name);
+
+	len = strlen (prefix) + strlen (maildir) + strlen (user_name) + 2;
+	mailfile = xmalloc (len);
+
+	if (prefix[0]) {
+		(void) snprintf (mailfile, len, "%s/%s/%s",
+	    	             prefix, maildir, user_name);
+	}
+	else {
+		(void) snprintf (mailfile, len, "%s/%s",
+	    	             maildir, user_name);
+	}
+	mailfile[len-1] = '\0';
 
 	if (access (mailfile, F_OK) != 0) {
 		if (ENOENT == errno) {
 			fprintf (stderr,
 			         _("%s: %s mail spool (%s) not found\n"),
 			         Prog, user_name, mailfile);
+			free(mailfile);
 			return 0;
 		} else {
 			fprintf (stderr,
@@ -846,6 +868,7 @@ static int remove_mailbox (void)
 			              user_name, (unsigned int) user_id,
 			              SHADOW_AUDIT_FAILURE);
 #endif				/* WITH_AUDIT */
+			free(mailfile);
 			return -1;
 		}
 	}
@@ -874,6 +897,7 @@ static int remove_mailbox (void)
 			              SHADOW_AUDIT_SUCCESS);
 		}
 #endif				/* WITH_AUDIT */
+		free(mailfile);
 		return errors;
 	}
 	i = is_owner (user_id, mailfile);
@@ -890,8 +914,10 @@ static int remove_mailbox (void)
 		              user_name, (unsigned int) user_id,
 		              SHADOW_AUDIT_FAILURE);
 #endif				/* WITH_AUDIT */
+		free(mailfile);
 		return 1;
 	} else if (i == -1) {
+		free(mailfile);
 		return 0;		/* mailbox doesn't exist */
 	}
 	if (unlink (mailfile) != 0) {
@@ -917,6 +943,7 @@ static int remove_mailbox (void)
 		              SHADOW_AUDIT_SUCCESS);
 	}
 #endif				/* WITH_AUDIT */
+	free(mailfile);
 	return errors;
 }
 
@@ -925,7 +952,7 @@ static int remove_tcbdir (const char *user_name, uid_t user_id)
 {
 	char *buf;
 	int ret = 0;
-	size_t bufsize = (sizeof TCB_DIR) + strlen (user_name) + 2;
+	size_t buflen = (sizeof TCB_DIR) + strlen (user_name) + 2;
 
 	if (!getdef_bool ("USE_TCB")) {
 		return 0;
@@ -990,6 +1017,7 @@ int main (int argc, char **argv)
 	(void) textdomain (PACKAGE);
 
 	process_root_flag ("-R", argc, argv);
+	prefix = process_prefix_flag ("-P", argc, argv);
 
 	OPENLOG ("userdel");
 #ifdef WITH_AUDIT
@@ -1006,6 +1034,7 @@ int main (int argc, char **argv)
 			{"help",         no_argument,       NULL, 'h'},
 			{"remove",       no_argument,       NULL, 'r'},
 			{"root",         required_argument, NULL, 'R'},
+			{"prefix",       required_argument, NULL, 'P'},
 #ifdef WITH_SELINUX
 			{"selinux-user", no_argument,       NULL, 'Z'},
 #endif				/* WITH_SELINUX */
@@ -1013,9 +1042,9 @@ int main (int argc, char **argv)
 		};
 		while ((c = getopt_long (argc, argv,
 #ifdef WITH_SELINUX             
-		                         "fhrR:Z",
+		                         "fhrR:P:Z",
 #else				/* !WITH_SELINUX */
-		                         "fhrR:",
+		                         "fhrR:P:",
 #endif				/* !WITH_SELINUX */
 		                         long_options, NULL)) != -1) {
 			switch (c) {
@@ -1029,9 +1058,18 @@ int main (int argc, char **argv)
 				rflg = true;
 				break;
 			case 'R': /* no-op, handled in process_root_flag () */
+				Rflg = true;
+				break;
+			case 'P': /* no-op, handled in process_prefix_flag () */
 				break;
 #ifdef WITH_SELINUX             
 			case 'Z':
+				if (prefix[0]) {
+					fprintf (stderr,
+					         _("%s: -Z cannot be used with --prefix\n"),
+					         Prog);
+					exit (E_BAD_ARG);
+				}
 				if (is_selinux_enabled () > 0) {
 					Zflg = true;
 				} else {
@@ -1103,9 +1141,12 @@ int main (int argc, char **argv)
 	 */
 	user_name = argv[argc - 1];
 	{
-		struct passwd *pwd;
-		pwd = getpwnam (user_name); /* local, no need for xgetpwnam */
+		const struct passwd *pwd;
+
+		pw_open(O_RDONLY);
+		pwd = pw_locate (user_name); /* we care only about local users */
 		if (NULL == pwd) {
+			pw_close();
 			fprintf (stderr, _("%s: user '%s' does not exist\n"),
 				 Prog, user_name);
 #ifdef WITH_AUDIT
@@ -1118,7 +1159,19 @@ int main (int argc, char **argv)
 		}
 		user_id = pwd->pw_uid;
 		user_gid = pwd->pw_gid;
-		user_home = xstrdup (pwd->pw_dir);
+		
+		if(prefix[0]) {
+		
+			size_t len = strlen(prefix) + strlen(pwd->pw_dir) + 2;
+			int wlen;
+			user_home = xmalloc(len);
+			wlen = snprintf(user_home, len, "%s/%s", prefix, pwd->pw_dir);
+			assert (wlen == (int) len -1);
+		}
+		else {
+			user_home = xstrdup (pwd->pw_dir);
+		}
+		pw_close();
 	}
 #ifdef WITH_TCB
 	if (shadowtcb_set_user (user_name) == SHADOWTCB_FAILURE) {
@@ -1150,7 +1203,7 @@ int main (int argc, char **argv)
 	 * Note: This is a best effort basis. The user may log in between,
 	 * a cron job may be started on her behalf, etc.
 	 */
-	if (user_busy (user_name, user_id) != 0) {
+	if ((prefix[0] == '\0') && !Rflg && user_busy (user_name, user_id) != 0) {
 		if (!fflg) {
 #ifdef WITH_AUDIT
 			audit_logger (AUDIT_DEL_USER, Prog,
@@ -1201,8 +1254,8 @@ int main (int argc, char **argv)
 		 * prevent accidents if someone has /home or / as home
 		 * directory...  --marekm
 		 */
-		setpwent ();
-		while ((pwd = getpwent ())) {
+		prefix_setpwent ();
+		while ((pwd = prefix_getpwent ())) {
 			if (strcmp (pwd->pw_name, user_name) == 0) {
 				continue;
 			}
@@ -1216,11 +1269,28 @@ int main (int argc, char **argv)
 				break;
 			}
 		}
-		endpwent ();
+		prefix_endpwent ();
 	}
 #endif				/* EXTRA_CHECK_HOME_DIR */
 
 	if (rflg) {
+#ifdef WITH_BTRFS
+		int is_subvolume = btrfs_is_subvolume (user_home);
+		if (is_subvolume < 0) {
+		    errors++;
+		    /* continue */
+		}
+		else if (is_subvolume > 0) {
+			if (btrfs_remove_subvolume (user_home)) {
+				fprintf (stderr,
+				         _("%s: error removing subvolume %s\n"),
+				         Prog, user_home);
+				errors++;
+				/* continue */
+			}
+		}
+		else
+#endif
 		if (remove_tree (user_home, true) != 0) {
 			fprintf (stderr,
 			         _("%s: error removing directory %s\n"),
@@ -1268,7 +1338,8 @@ int main (int argc, char **argv)
 	 * Cancel any crontabs or at jobs. Have to do this before we remove
 	 * the entry from /etc/passwd.
 	 */
-	user_cancel (user_name);
+	if(prefix[0] == '\0')
+		user_cancel (user_name);
 	close_files ();
 
 #ifdef WITH_TCB
@@ -1277,6 +1348,7 @@ int main (int argc, char **argv)
 
 	nscd_flush_cache ("passwd");
 	nscd_flush_cache ("group");
+	sssd_flush_cache (SSSD_DB_PASSWD | SSSD_DB_GROUP);
 
 	return ((0 != errors) ? E_HOMEDIR : E_SUCCESS);
 }
